@@ -3,6 +3,7 @@
 
 const int tensor_split_max = 16;
 const int images_max = 8;
+const int audio_max = 4;
 const int logprobs_max = 5;
 
 // match kobold's sampler list and order
@@ -83,6 +84,7 @@ struct generation_inputs
     const char * negative_prompt = nullptr;
     const float guidance_scale = 1;
     const char * images[images_max] = {};
+    const char * audio[audio_max] = {};
     const int max_context_length = 0;
     const int max_length = 0;
     const float temperature = 0.0f;
@@ -161,6 +163,7 @@ struct sd_load_model_inputs
     const char * vulkan_info = nullptr;
     const int threads = 0;
     const int quant = 0;
+    const bool flash_attention = false;
     const bool taesd = false;
     const int tiled_vae_threshold = 0;
     const char * t5xxl_filename = nullptr;
@@ -284,6 +287,7 @@ extern std::string mmproj_filename;
 extern std::string draftmodel_filename;
 extern std::vector<std::string> generated_tokens;
 extern bool generation_finished;
+extern bool audio_multimodal_supported;
 extern float last_eval_time;
 extern float last_process_time;
 extern int last_token_count;
